@@ -1,4 +1,4 @@
-/* $OpenBSD: dh.c,v 1.25 2014/07/14 00:35:10 deraadt Exp $ */
+/* $OpenBSD: dh.c,v 1.1 2014/08/26 17:47:24 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -168,7 +168,6 @@ bad:
 #endif
 		goto end;
 	}
-	ERR_load_crypto_strings();
 
 #ifndef OPENSSL_NO_ENGINE
 	setup_engine(bio_err, engine, 0);
