@@ -1,4 +1,4 @@
-/* $OpenBSD: err.c,v 1.37 2014/07/10 22:45:57 jsing Exp $ */
+/* $OpenBSD: err.c,v 1.39 2014/08/24 16:11:39 bcook Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -596,7 +596,7 @@ build_SYS_str_reasons(void)
 		if (str->string == NULL) {
 			char (*dest)[LEN_SYS_STR_REASON] =
 			    &(strerror_tab[i - 1]);
-			char *src = strerror(i);
+			const char *src = strerror(i);
 			if (src != NULL) {
 				strlcpy(*dest, src, sizeof *dest);
 				str->string = *dest;
@@ -1105,8 +1105,9 @@ ERR_add_error_vdata(int num, va_list args)
 {
 	char format[129];
 	char *errbuf;
-	format[0] = '\0';
 	int i;
+
+	format[0] = '\0';
 	for (i = 0; i < num; i++) {
 		if (strlcat(format, "%s", sizeof(format)) >= sizeof(format)) {
 			ERR_set_error_data("too many errors", ERR_TXT_STRING);
