@@ -19,9 +19,14 @@
 #elif defined(__linux__)
 #include <endian.h>
 
-#elif defined(__sun)
+#elif defined(__sun) || defined(_AIX) || defined(__hpux)
+#include <sys/types.h>
 #include <arpa/nameser_compat.h>
 
+#elif defined(__sgi)
+#include <standards.h>
+#include <sys/endian.h>
+
 #else
 #include_next <machine/endian.h>
 
